/* DO NOT EDIT! GENERATED AUTOMATICALLY! */
/* Test the Unicode character type functions.
   Copyright (C) 2007-2025 Free Software Foundation, Inc.

   This file is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published
   by the Free Software Foundation, either version 3 of the License,
   or (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */

#include "test-predicate-part1.h"

    { 0x0028, 0x0028 },
    { 0x005B, 0x005B },
    { 0x007B, 0x007B },
    { 0x0F3A, 0x0F3A },
    { 0x0F3C, 0x0F3C },
    { 0x169B, 0x169B },
    { 0x201A, 0x201A },
    { 0x201E, 0x201E },
    { 0x2045, 0x2045 },
    { 0x207D, 0x207D },
    { 0x208D, 0x208D },
    { 0x2308, 0x2308 },
    { 0x230A, 0x230A },
    { 0x2329, 0x2329 },
    { 0x2768, 0x2768 },
    { 0x276A, 0x276A },
    { 0x276C, 0x276C },
    { 0x276E, 0x276E },
    { 0x2770, 0x2770 },
    { 0x2772, 0x2772 },
    { 0x2774, 0x2774 },
    { 0x27C5, 0x27C5 },
    { 0x27E6, 0x27E6 },
    { 0x27E8, 0x27E8 },
    { 0x27EA, 0x27EA },
    { 0x27EC, 0x27EC },
    { 0x27EE, 0x27EE },
    { 0x2983, 0x2983 },
    { 0x2985, 0x2985 },
    { 0x2987, 0x2987 },
    { 0x2989, 0x2989 },
    { 0x298B, 0x298B },
    { 0x298D, 0x298D },
    { 0x298F, 0x298F },
    { 0x2991, 0x2991 },
    { 0x2993, 0x2993 },
    { 0x2995, 0x2995 },
    { 0x2997, 0x2997 },
    { 0x29D8, 0x29D8 },
    { 0x29DA, 0x29DA },
    { 0x29FC, 0x29FC },
    { 0x2E22, 0x2E22 },
    { 0x2E24, 0x2E24 },
    { 0x2E26, 0x2E26 },
    { 0x2E28, 0x2E28 },
    { 0x2E42, 0x2E42 },
    { 0x2E55, 0x2E55 },
    { 0x2E57, 0x2E57 },
    { 0x2E59, 0x2E59 },
    { 0x2E5B, 0x2E5B },
    { 0x3008, 0x3008 },
    { 0x300A, 0x300A },
    { 0x300C, 0x300C },
    { 0x300E, 0x300E },
    { 0x3010, 0x3010 },
    { 0x3014, 0x3014 },
    { 0x3016, 0x3016 },
    { 0x3018, 0x3018 },
    { 0x301A, 0x301A },
    { 0x301D, 0x301D },
    { 0xFD3F, 0xFD3F },
    { 0xFE17, 0xFE17 },
    { 0xFE35, 0xFE35 },
    { 0xFE37, 0xFE37 },
    { 0xFE39, 0xFE39 },
    { 0xFE3B, 0xFE3B },
    { 0xFE3D, 0xFE3D },
    { 0xFE3F, 0xFE3F },
    { 0xFE41, 0xFE41 },
    { 0xFE43, 0xFE43 },
    { 0xFE47, 0xFE47 },
    { 0xFE59, 0xFE59 },
    { 0xFE5B, 0xFE5B },
    { 0xFE5D, 0xFE5D },
    { 0xFF08, 0xFF08 },
    { 0xFF3B, 0xFF3B },
    { 0xFF5B, 0xFF5B },
    { 0xFF5F, 0xFF5F },
    { 0xFF62, 0xFF62 }

#define PREDICATE(c) uc_is_general_category (c, UC_CATEGORY_Ps)
#include "test-predicate-part2.h"
