/* DO NOT EDIT! GENERATED AUTOMATICALLY! */
/* Arabic joining type of Unicode characters.  */
/* Generated automatically by gen-uni-tables.c for Unicode 17.0.0.  */

/* Copyright (C) 2000-2025 Free Software Foundation, Inc.

   This file is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published
   by the Free Software Foundation, either version 3 of the License,
   or (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */

    { 0x0600, UC_JOINING_TYPE_U },
    { 0x0601, UC_JOINING_TYPE_U },
    { 0x0602, UC_JOINING_TYPE_U },
    { 0x0603, UC_JOINING_TYPE_U },
    { 0x0604, UC_JOINING_TYPE_U },
    { 0x0605, UC_JOINING_TYPE_U },
    { 0x0608, UC_JOINING_TYPE_U },
    { 0x060B, UC_JOINING_TYPE_U },
    { 0x0620, UC_JOINING_TYPE_D },
    { 0x0621, UC_JOINING_TYPE_U },
    { 0x0622, UC_JOINING_TYPE_R },
    { 0x0623, UC_JOINING_TYPE_R },
    { 0x0624, UC_JOINING_TYPE_R },
    { 0x0625, UC_JOINING_TYPE_R },
    { 0x0626, UC_JOINING_TYPE_D },
    { 0x0627, UC_JOINING_TYPE_R },
    { 0x0628, UC_JOINING_TYPE_D },
    { 0x0629, UC_JOINING_TYPE_R },
    { 0x062A, UC_JOINING_TYPE_D },
    { 0x062B, UC_JOINING_TYPE_D },
    { 0x062C, UC_JOINING_TYPE_D },
    { 0x062D, UC_JOINING_TYPE_D },
    { 0x062E, UC_JOINING_TYPE_D },
    { 0x062F, UC_JOINING_TYPE_R },
    { 0x0630, UC_JOINING_TYPE_R },
    { 0x0631, UC_JOINING_TYPE_R },
    { 0x0632, UC_JOINING_TYPE_R },
    { 0x0633, UC_JOINING_TYPE_D },
    { 0x0634, UC_JOINING_TYPE_D },
    { 0x0635, UC_JOINING_TYPE_D },
    { 0x0636, UC_JOINING_TYPE_D },
    { 0x0637, UC_JOINING_TYPE_D },
    { 0x0638, UC_JOINING_TYPE_D },
    { 0x0639, UC_JOINING_TYPE_D },
    { 0x063A, UC_JOINING_TYPE_D },
    { 0x063B, UC_JOINING_TYPE_D },
    { 0x063C, UC_JOINING_TYPE_D },
    { 0x063D, UC_JOINING_TYPE_D },
    { 0x063E, UC_JOINING_TYPE_D },
    { 0x063F, UC_JOINING_TYPE_D },
    { 0x0640, UC_JOINING_TYPE_C },
    { 0x0641, UC_JOINING_TYPE_D },
    { 0x0642, UC_JOINING_TYPE_D },
    { 0x0643, UC_JOINING_TYPE_D },
    { 0x0644, UC_JOINING_TYPE_D },
    { 0x0645, UC_JOINING_TYPE_D },
    { 0x0646, UC_JOINING_TYPE_D },
    { 0x0647, UC_JOINING_TYPE_D },
    { 0x0648, UC_JOINING_TYPE_R },
    { 0x0649, UC_JOINING_TYPE_D },
    { 0x064A, UC_JOINING_TYPE_D },
    { 0x066E, UC_JOINING_TYPE_D },
    { 0x066F, UC_JOINING_TYPE_D },
    { 0x0671, UC_JOINING_TYPE_R },
    { 0x0672, UC_JOINING_TYPE_R },
    { 0x0673, UC_JOINING_TYPE_R },
    { 0x0674, UC_JOINING_TYPE_U },
    { 0x0675, UC_JOINING_TYPE_R },
    { 0x0676, UC_JOINING_TYPE_R },
    { 0x0677, UC_JOINING_TYPE_R },
    { 0x0678, UC_JOINING_TYPE_D },
    { 0x0679, UC_JOINING_TYPE_D },
    { 0x067A, UC_JOINING_TYPE_D },
    { 0x067B, UC_JOINING_TYPE_D },
    { 0x067C, UC_JOINING_TYPE_D },
    { 0x067D, UC_JOINING_TYPE_D },
    { 0x067E, UC_JOINING_TYPE_D },
    { 0x067F, UC_JOINING_TYPE_D },
    { 0x0680, UC_JOINING_TYPE_D },
    { 0x0681, UC_JOINING_TYPE_D },
    { 0x0682, UC_JOINING_TYPE_D },
    { 0x0683, UC_JOINING_TYPE_D },
    { 0x0684, UC_JOINING_TYPE_D },
    { 0x0685, UC_JOINING_TYPE_D },
    { 0x0686, UC_JOINING_TYPE_D },
    { 0x0687, UC_JOINING_TYPE_D },
    { 0x0688, UC_JOINING_TYPE_R },
    { 0x0689, UC_JOINING_TYPE_R },
    { 0x068A, UC_JOINING_TYPE_R },
    { 0x068B, UC_JOINING_TYPE_R },
    { 0x068C, UC_JOINING_TYPE_R },
    { 0x068D, UC_JOINING_TYPE_R },
    { 0x068E, UC_JOINING_TYPE_R },
    { 0x068F, UC_JOINING_TYPE_R },
    { 0x0690, UC_JOINING_TYPE_R },
    { 0x0691, UC_JOINING_TYPE_R },
    { 0x0692, UC_JOINING_TYPE_R },
    { 0x0693, UC_JOINING_TYPE_R },
    { 0x0694, UC_JOINING_TYPE_R },
    { 0x0695, UC_JOINING_TYPE_R },
    { 0x0696, UC_JOINING_TYPE_R },
    { 0x0697, UC_JOINING_TYPE_R },
    { 0x0698, UC_JOINING_TYPE_R },
    { 0x0699, UC_JOINING_TYPE_R },
    { 0x069A, UC_JOINING_TYPE_D },
    { 0x069B, UC_JOINING_TYPE_D },
    { 0x069C, UC_JOINING_TYPE_D },
    { 0x069D, UC_JOINING_TYPE_D },
    { 0x069E, UC_JOINING_TYPE_D },
    { 0x069F, UC_JOINING_TYPE_D },
    { 0x06A0, UC_JOINING_TYPE_D },
    { 0x06A1, UC_JOINING_TYPE_D },
    { 0x06A2, UC_JOINING_TYPE_D },
    { 0x06A3, UC_JOINING_TYPE_D },
    { 0x06A4, UC_JOINING_TYPE_D },
    { 0x06A5, UC_JOINING_TYPE_D },
    { 0x06A6, UC_JOINING_TYPE_D },
    { 0x06A7, UC_JOINING_TYPE_D },
    { 0x06A8, UC_JOINING_TYPE_D },
    { 0x06A9, UC_JOINING_TYPE_D },
    { 0x06AA, UC_JOINING_TYPE_D },
    { 0x06AB, UC_JOINING_TYPE_D },
    { 0x06AC, UC_JOINING_TYPE_D },
    { 0x06AD, UC_JOINING_TYPE_D },
    { 0x06AE, UC_JOINING_TYPE_D },
    { 0x06AF, UC_JOINING_TYPE_D },
    { 0x06B0, UC_JOINING_TYPE_D },
    { 0x06B1, UC_JOINING_TYPE_D },
    { 0x06B2, UC_JOINING_TYPE_D },
    { 0x06B3, UC_JOINING_TYPE_D },
    { 0x06B4, UC_JOINING_TYPE_D },
    { 0x06B5, UC_JOINING_TYPE_D },
    { 0x06B6, UC_JOINING_TYPE_D },
    { 0x06B7, UC_JOINING_TYPE_D },
    { 0x06B8, UC_JOINING_TYPE_D },
    { 0x06B9, UC_JOINING_TYPE_D },
    { 0x06BA, UC_JOINING_TYPE_D },
    { 0x06BB, UC_JOINING_TYPE_D },
    { 0x06BC, UC_JOINING_TYPE_D },
    { 0x06BD, UC_JOINING_TYPE_D },
    { 0x06BE, UC_JOINING_TYPE_D },
    { 0x06BF, UC_JOINING_TYPE_D },
    { 0x06C0, UC_JOINING_TYPE_R },
    { 0x06C1, UC_JOINING_TYPE_D },
    { 0x06C2, UC_JOINING_TYPE_D },
    { 0x06C3, UC_JOINING_TYPE_R },
    { 0x06C4, UC_JOINING_TYPE_R },
    { 0x06C5, UC_JOINING_TYPE_R },
    { 0x06C6, UC_JOINING_TYPE_R },
    { 0x06C7, UC_JOINING_TYPE_R },
    { 0x06C8, UC_JOINING_TYPE_R },
    { 0x06C9, UC_JOINING_TYPE_R },
    { 0x06CA, UC_JOINING_TYPE_R },
    { 0x06CB, UC_JOINING_TYPE_R },
    { 0x06CC, UC_JOINING_TYPE_D },
    { 0x06CD, UC_JOINING_TYPE_R },
    { 0x06CE, UC_JOINING_TYPE_D },
    { 0x06CF, UC_JOINING_TYPE_R },
    { 0x06D0, UC_JOINING_TYPE_D },
    { 0x06D1, UC_JOINING_TYPE_D },
    { 0x06D2, UC_JOINING_TYPE_R },
    { 0x06D3, UC_JOINING_TYPE_R },
    { 0x06D5, UC_JOINING_TYPE_R },
    { 0x06DD, UC_JOINING_TYPE_U },
    { 0x06EE, UC_JOINING_TYPE_R },
    { 0x06EF, UC_JOINING_TYPE_R },
    { 0x06FA, UC_JOINING_TYPE_D },
    { 0x06FB, UC_JOINING_TYPE_D },
    { 0x06FC, UC_JOINING_TYPE_D },
    { 0x06FF, UC_JOINING_TYPE_D },
    { 0x070F, UC_JOINING_TYPE_T },
    { 0x0710, UC_JOINING_TYPE_R },
    { 0x0712, UC_JOINING_TYPE_D },
    { 0x0713, UC_JOINING_TYPE_D },
    { 0x0714, UC_JOINING_TYPE_D },
    { 0x0715, UC_JOINING_TYPE_R },
    { 0x0716, UC_JOINING_TYPE_R },
    { 0x0717, UC_JOINING_TYPE_R },
    { 0x0718, UC_JOINING_TYPE_R },
    { 0x0719, UC_JOINING_TYPE_R },
    { 0x071A, UC_JOINING_TYPE_D },
    { 0x071B, UC_JOINING_TYPE_D },
    { 0x071C, UC_JOINING_TYPE_D },
    { 0x071D, UC_JOINING_TYPE_D },
    { 0x071E, UC_JOINING_TYPE_R },
    { 0x071F, UC_JOINING_TYPE_D },
    { 0x0720, UC_JOINING_TYPE_D },
    { 0x0721, UC_JOINING_TYPE_D },
    { 0x0722, UC_JOINING_TYPE_D },
    { 0x0723, UC_JOINING_TYPE_D },
    { 0x0724, UC_JOINING_TYPE_D },
    { 0x0725, UC_JOINING_TYPE_D },
    { 0x0726, UC_JOINING_TYPE_D },
    { 0x0727, UC_JOINING_TYPE_D },
    { 0x0728, UC_JOINING_TYPE_R },
    { 0x0729, UC_JOINING_TYPE_D },
    { 0x072A, UC_JOINING_TYPE_R },
    { 0x072B, UC_JOINING_TYPE_D },
    { 0x072C, UC_JOINING_TYPE_R },
    { 0x072D, UC_JOINING_TYPE_D },
    { 0x072E, UC_JOINING_TYPE_D },
    { 0x072F, UC_JOINING_TYPE_R },
    { 0x074D, UC_JOINING_TYPE_R },
    { 0x074E, UC_JOINING_TYPE_D },
    { 0x074F, UC_JOINING_TYPE_D },
    { 0x0750, UC_JOINING_TYPE_D },
    { 0x0751, UC_JOINING_TYPE_D },
    { 0x0752, UC_JOINING_TYPE_D },
    { 0x0753, UC_JOINING_TYPE_D },
    { 0x0754, UC_JOINING_TYPE_D },
    { 0x0755, UC_JOINING_TYPE_D },
    { 0x0756, UC_JOINING_TYPE_D },
    { 0x0757, UC_JOINING_TYPE_D },
    { 0x0758, UC_JOINING_TYPE_D },
    { 0x0759, UC_JOINING_TYPE_R },
    { 0x075A, UC_JOINING_TYPE_R },
    { 0x075B, UC_JOINING_TYPE_R },
    { 0x075C, UC_JOINING_TYPE_D },
    { 0x075D, UC_JOINING_TYPE_D },
    { 0x075E, UC_JOINING_TYPE_D },
    { 0x075F, UC_JOINING_TYPE_D },
    { 0x0760, UC_JOINING_TYPE_D },
    { 0x0761, UC_JOINING_TYPE_D },
    { 0x0762, UC_JOINING_TYPE_D },
    { 0x0763, UC_JOINING_TYPE_D },
    { 0x0764, UC_JOINING_TYPE_D },
    { 0x0765, UC_JOINING_TYPE_D },
    { 0x0766, UC_JOINING_TYPE_D },
    { 0x0767, UC_JOINING_TYPE_D },
    { 0x0768, UC_JOINING_TYPE_D },
    { 0x0769, UC_JOINING_TYPE_D },
    { 0x076A, UC_JOINING_TYPE_D },
    { 0x076B, UC_JOINING_TYPE_R },
    { 0x076C, UC_JOINING_TYPE_R },
    { 0x076D, UC_JOINING_TYPE_D },
    { 0x076E, UC_JOINING_TYPE_D },
    { 0x076F, UC_JOINING_TYPE_D },
    { 0x0770, UC_JOINING_TYPE_D },
    { 0x0771, UC_JOINING_TYPE_R },
    { 0x0772, UC_JOINING_TYPE_D },
    { 0x0773, UC_JOINING_TYPE_R },
    { 0x0774, UC_JOINING_TYPE_R },
    { 0x0775, UC_JOINING_TYPE_D },
    { 0x0776, UC_JOINING_TYPE_D },
    { 0x0777, UC_JOINING_TYPE_D },
    { 0x0778, UC_JOINING_TYPE_R },
    { 0x0779, UC_JOINING_TYPE_R },
    { 0x077A, UC_JOINING_TYPE_D },
    { 0x077B, UC_JOINING_TYPE_D },
    { 0x077C, UC_JOINING_TYPE_D },
    { 0x077D, UC_JOINING_TYPE_D },
    { 0x077E, UC_JOINING_TYPE_D },
    { 0x077F, UC_JOINING_TYPE_D },
    { 0x07CA, UC_JOINING_TYPE_D },
    { 0x07CB, UC_JOINING_TYPE_D },
    { 0x07CC, UC_JOINING_TYPE_D },
    { 0x07CD, UC_JOINING_TYPE_D },
    { 0x07CE, UC_JOINING_TYPE_D },
    { 0x07CF, UC_JOINING_TYPE_D },
    { 0x07D0, UC_JOINING_TYPE_D },
    { 0x07D1, UC_JOINING_TYPE_D },
    { 0x07D2, UC_JOINING_TYPE_D },
    { 0x07D3, UC_JOINING_TYPE_D },
    { 0x07D4, UC_JOINING_TYPE_D },
    { 0x07D5, UC_JOINING_TYPE_D },
    { 0x07D6, UC_JOINING_TYPE_D },
    { 0x07D7, UC_JOINING_TYPE_D },
    { 0x07D8, UC_JOINING_TYPE_D },
    { 0x07D9, UC_JOINING_TYPE_D },
    { 0x07DA, UC_JOINING_TYPE_D },
    { 0x07DB, UC_JOINING_TYPE_D },
    { 0x07DC, UC_JOINING_TYPE_D },
    { 0x07DD, UC_JOINING_TYPE_D },
    { 0x07DE, UC_JOINING_TYPE_D },
    { 0x07DF, UC_JOINING_TYPE_D },
    { 0x07E0, UC_JOINING_TYPE_D },
    { 0x07E1, UC_JOINING_TYPE_D },
    { 0x07E2, UC_JOINING_TYPE_D },
    { 0x07E3, UC_JOINING_TYPE_D },
    { 0x07E4, UC_JOINING_TYPE_D },
    { 0x07E5, UC_JOINING_TYPE_D },
    { 0x07E6, UC_JOINING_TYPE_D },
    { 0x07E7, UC_JOINING_TYPE_D },
    { 0x07E8, UC_JOINING_TYPE_D },
    { 0x07E9, UC_JOINING_TYPE_D },
    { 0x07EA, UC_JOINING_TYPE_D },
    { 0x07FA, UC_JOINING_TYPE_C },
    { 0x0840, UC_JOINING_TYPE_R },
    { 0x0841, UC_JOINING_TYPE_D },
    { 0x0842, UC_JOINING_TYPE_D },
    { 0x0843, UC_JOINING_TYPE_D },
    { 0x0844, UC_JOINING_TYPE_D },
    { 0x0845, UC_JOINING_TYPE_D },
    { 0x0846, UC_JOINING_TYPE_R },
    { 0x0847, UC_JOINING_TYPE_R },
    { 0x0848, UC_JOINING_TYPE_D },
    { 0x0849, UC_JOINING_TYPE_R },
    { 0x084A, UC_JOINING_TYPE_D },
    { 0x084B, UC_JOINING_TYPE_D },
    { 0x084C, UC_JOINING_TYPE_D },
    { 0x084D, UC_JOINING_TYPE_D },
    { 0x084E, UC_JOINING_TYPE_D },
    { 0x084F, UC_JOINING_TYPE_D },
    { 0x0850, UC_JOINING_TYPE_D },
    { 0x0851, UC_JOINING_TYPE_D },
    { 0x0852, UC_JOINING_TYPE_D },
    { 0x0853, UC_JOINING_TYPE_D },
    { 0x0854, UC_JOINING_TYPE_R },
    { 0x0855, UC_JOINING_TYPE_D },
    { 0x0856, UC_JOINING_TYPE_R },
    { 0x0857, UC_JOINING_TYPE_R },
    { 0x0858, UC_JOINING_TYPE_R },
    { 0x0860, UC_JOINING_TYPE_D },
    { 0x0861, UC_JOINING_TYPE_U },
    { 0x0862, UC_JOINING_TYPE_D },
    { 0x0863, UC_JOINING_TYPE_D },
    { 0x0864, UC_JOINING_TYPE_D },
    { 0x0865, UC_JOINING_TYPE_D },
    { 0x0866, UC_JOINING_TYPE_U },
    { 0x0867, UC_JOINING_TYPE_R },
    { 0x0868, UC_JOINING_TYPE_D },
    { 0x0869, UC_JOINING_TYPE_R },
    { 0x086A, UC_JOINING_TYPE_R },
    { 0x0870, UC_JOINING_TYPE_R },
    { 0x0871, UC_JOINING_TYPE_R },
    { 0x0872, UC_JOINING_TYPE_R },
    { 0x0873, UC_JOINING_TYPE_R },
    { 0x0874, UC_JOINING_TYPE_R },
    { 0x0875, UC_JOINING_TYPE_R },
    { 0x0876, UC_JOINING_TYPE_R },
    { 0x0877, UC_JOINING_TYPE_R },
    { 0x0878, UC_JOINING_TYPE_R },
    { 0x0879, UC_JOINING_TYPE_R },
    { 0x087A, UC_JOINING_TYPE_R },
    { 0x087B, UC_JOINING_TYPE_R },
    { 0x087C, UC_JOINING_TYPE_R },
    { 0x087D, UC_JOINING_TYPE_R },
    { 0x087E, UC_JOINING_TYPE_R },
    { 0x087F, UC_JOINING_TYPE_R },
    { 0x0880, UC_JOINING_TYPE_R },
    { 0x0881, UC_JOINING_TYPE_R },
    { 0x0882, UC_JOINING_TYPE_R },
    { 0x0883, UC_JOINING_TYPE_C },
    { 0x0884, UC_JOINING_TYPE_C },
    { 0x0885, UC_JOINING_TYPE_C },
    { 0x0886, UC_JOINING_TYPE_D },
    { 0x0887, UC_JOINING_TYPE_U },
    { 0x0888, UC_JOINING_TYPE_U },
    { 0x0889, UC_JOINING_TYPE_D },
    { 0x088A, UC_JOINING_TYPE_D },
    { 0x088B, UC_JOINING_TYPE_D },
    { 0x088C, UC_JOINING_TYPE_D },
    { 0x088D, UC_JOINING_TYPE_D },
    { 0x088E, UC_JOINING_TYPE_R },
    { 0x088F, UC_JOINING_TYPE_D },
    { 0x0890, UC_JOINING_TYPE_U },
    { 0x0891, UC_JOINING_TYPE_U },
    { 0x08A0, UC_JOINING_TYPE_D },
    { 0x08A1, UC_JOINING_TYPE_D },
    { 0x08A2, UC_JOINING_TYPE_D },
    { 0x08A3, UC_JOINING_TYPE_D },
    { 0x08A4, UC_JOINING_TYPE_D },
    { 0x08A5, UC_JOINING_TYPE_D },
    { 0x08A6, UC_JOINING_TYPE_D },
    { 0x08A7, UC_JOINING_TYPE_D },
    { 0x08A8, UC_JOINING_TYPE_D },
    { 0x08A9, UC_JOINING_TYPE_D },
    { 0x08AA, UC_JOINING_TYPE_R },
    { 0x08AB, UC_JOINING_TYPE_R },
    { 0x08AC, UC_JOINING_TYPE_R },
    { 0x08AD, UC_JOINING_TYPE_U },
    { 0x08AE, UC_JOINING_TYPE_R },
    { 0x08AF, UC_JOINING_TYPE_D },
    { 0x08B0, UC_JOINING_TYPE_D },
    { 0x08B1, UC_JOINING_TYPE_R },
    { 0x08B2, UC_JOINING_TYPE_R },
    { 0x08B3, UC_JOINING_TYPE_D },
    { 0x08B4, UC_JOINING_TYPE_D },
    { 0x08B5, UC_JOINING_TYPE_D },
    { 0x08B6, UC_JOINING_TYPE_D },
    { 0x08B7, UC_JOINING_TYPE_D },
    { 0x08B8, UC_JOINING_TYPE_D },
    { 0x08B9, UC_JOINING_TYPE_R },
    { 0x08BA, UC_JOINING_TYPE_D },
    { 0x08BB, UC_JOINING_TYPE_D },
    { 0x08BC, UC_JOINING_TYPE_D },
    { 0x08BD, UC_JOINING_TYPE_D },
    { 0x08BE, UC_JOINING_TYPE_D },
    { 0x08BF, UC_JOINING_TYPE_D },
    { 0x08C0, UC_JOINING_TYPE_D },
    { 0x08C1, UC_JOINING_TYPE_D },
    { 0x08C2, UC_JOINING_TYPE_D },
    { 0x08C3, UC_JOINING_TYPE_D },
    { 0x08C4, UC_JOINING_TYPE_D },
    { 0x08C5, UC_JOINING_TYPE_D },
    { 0x08C6, UC_JOINING_TYPE_D },
    { 0x08C7, UC_JOINING_TYPE_D },
    { 0x08C8, UC_JOINING_TYPE_D },
    { 0x08E2, UC_JOINING_TYPE_U },
    { 0x1806, UC_JOINING_TYPE_U },
    { 0x1807, UC_JOINING_TYPE_D },
    { 0x180A, UC_JOINING_TYPE_C },
    { 0x180E, UC_JOINING_TYPE_U },
    { 0x1820, UC_JOINING_TYPE_D },
    { 0x1821, UC_JOINING_TYPE_D },
    { 0x1822, UC_JOINING_TYPE_D },
    { 0x1823, UC_JOINING_TYPE_D },
    { 0x1824, UC_JOINING_TYPE_D },
    { 0x1825, UC_JOINING_TYPE_D },
    { 0x1826, UC_JOINING_TYPE_D },
    { 0x1827, UC_JOINING_TYPE_D },
    { 0x1828, UC_JOINING_TYPE_D },
    { 0x1829, UC_JOINING_TYPE_D },
    { 0x182A, UC_JOINING_TYPE_D },
    { 0x182B, UC_JOINING_TYPE_D },
    { 0x182C, UC_JOINING_TYPE_D },
    { 0x182D, UC_JOINING_TYPE_D },
    { 0x182E, UC_JOINING_TYPE_D },
    { 0x182F, UC_JOINING_TYPE_D },
    { 0x1830, UC_JOINING_TYPE_D },
    { 0x1831, UC_JOINING_TYPE_D },
    { 0x1832, UC_JOINING_TYPE_D },
    { 0x1833, UC_JOINING_TYPE_D },
    { 0x1834, UC_JOINING_TYPE_D },
    { 0x1835, UC_JOINING_TYPE_D },
    { 0x1836, UC_JOINING_TYPE_D },
    { 0x1837, UC_JOINING_TYPE_D },
    { 0x1838, UC_JOINING_TYPE_D },
    { 0x1839, UC_JOINING_TYPE_D },
    { 0x183A, UC_JOINING_TYPE_D },
    { 0x183B, UC_JOINING_TYPE_D },
    { 0x183C, UC_JOINING_TYPE_D },
    { 0x183D, UC_JOINING_TYPE_D },
    { 0x183E, UC_JOINING_TYPE_D },
    { 0x183F, UC_JOINING_TYPE_D },
    { 0x1840, UC_JOINING_TYPE_D },
    { 0x1841, UC_JOINING_TYPE_D },
    { 0x1842, UC_JOINING_TYPE_D },
    { 0x1843, UC_JOINING_TYPE_D },
    { 0x1844, UC_JOINING_TYPE_D },
    { 0x1845, UC_JOINING_TYPE_D },
    { 0x1846, UC_JOINING_TYPE_D },
    { 0x1847, UC_JOINING_TYPE_D },
    { 0x1848, UC_JOINING_TYPE_D },
    { 0x1849, UC_JOINING_TYPE_D },
    { 0x184A, UC_JOINING_TYPE_D },
    { 0x184B, UC_JOINING_TYPE_D },
    { 0x184C, UC_JOINING_TYPE_D },
    { 0x184D, UC_JOINING_TYPE_D },
    { 0x184E, UC_JOINING_TYPE_D },
    { 0x184F, UC_JOINING_TYPE_D },
    { 0x1850, UC_JOINING_TYPE_D },
    { 0x1851, UC_JOINING_TYPE_D },
    { 0x1852, UC_JOINING_TYPE_D },
    { 0x1853, UC_JOINING_TYPE_D },
    { 0x1854, UC_JOINING_TYPE_D },
    { 0x1855, UC_JOINING_TYPE_D },
    { 0x1856, UC_JOINING_TYPE_D },
    { 0x1857, UC_JOINING_TYPE_D },
    { 0x1858, UC_JOINING_TYPE_D },
    { 0x1859, UC_JOINING_TYPE_D },
    { 0x185A, UC_JOINING_TYPE_D },
    { 0x185B, UC_JOINING_TYPE_D },
    { 0x185C, UC_JOINING_TYPE_D },
    { 0x185D, UC_JOINING_TYPE_D },
    { 0x185E, UC_JOINING_TYPE_D },
    { 0x185F, UC_JOINING_TYPE_D },
    { 0x1860, UC_JOINING_TYPE_D },
    { 0x1861, UC_JOINING_TYPE_D },
    { 0x1862, UC_JOINING_TYPE_D },
    { 0x1863, UC_JOINING_TYPE_D },
    { 0x1864, UC_JOINING_TYPE_D },
    { 0x1865, UC_JOINING_TYPE_D },
    { 0x1866, UC_JOINING_TYPE_D },
    { 0x1867, UC_JOINING_TYPE_D },
    { 0x1868, UC_JOINING_TYPE_D },
    { 0x1869, UC_JOINING_TYPE_D },
    { 0x186A, UC_JOINING_TYPE_D },
    { 0x186B, UC_JOINING_TYPE_D },
    { 0x186C, UC_JOINING_TYPE_D },
    { 0x186D, UC_JOINING_TYPE_D },
    { 0x186E, UC_JOINING_TYPE_D },
    { 0x186F, UC_JOINING_TYPE_D },
    { 0x1870, UC_JOINING_TYPE_D },
    { 0x1871, UC_JOINING_TYPE_D },
    { 0x1872, UC_JOINING_TYPE_D },
    { 0x1873, UC_JOINING_TYPE_D },
    { 0x1874, UC_JOINING_TYPE_D },
    { 0x1875, UC_JOINING_TYPE_D },
    { 0x1876, UC_JOINING_TYPE_D },
    { 0x1877, UC_JOINING_TYPE_D },
    { 0x1878, UC_JOINING_TYPE_D },
    { 0x1880, UC_JOINING_TYPE_U },
    { 0x1881, UC_JOINING_TYPE_U },
    { 0x1882, UC_JOINING_TYPE_U },
    { 0x1883, UC_JOINING_TYPE_U },
    { 0x1884, UC_JOINING_TYPE_U },
    { 0x1885, UC_JOINING_TYPE_T },
    { 0x1886, UC_JOINING_TYPE_T },
    { 0x1887, UC_JOINING_TYPE_D },
    { 0x1888, UC_JOINING_TYPE_D },
    { 0x1889, UC_JOINING_TYPE_D },
    { 0x188A, UC_JOINING_TYPE_D },
    { 0x188B, UC_JOINING_TYPE_D },
    { 0x188C, UC_JOINING_TYPE_D },
    { 0x188D, UC_JOINING_TYPE_D },
    { 0x188E, UC_JOINING_TYPE_D },
    { 0x188F, UC_JOINING_TYPE_D },
    { 0x1890, UC_JOINING_TYPE_D },
    { 0x1891, UC_JOINING_TYPE_D },
    { 0x1892, UC_JOINING_TYPE_D },
    { 0x1893, UC_JOINING_TYPE_D },
    { 0x1894, UC_JOINING_TYPE_D },
    { 0x1895, UC_JOINING_TYPE_D },
    { 0x1896, UC_JOINING_TYPE_D },
    { 0x1897, UC_JOINING_TYPE_D },
    { 0x1898, UC_JOINING_TYPE_D },
    { 0x1899, UC_JOINING_TYPE_D },
    { 0x189A, UC_JOINING_TYPE_D },
    { 0x189B, UC_JOINING_TYPE_D },
    { 0x189C, UC_JOINING_TYPE_D },
    { 0x189D, UC_JOINING_TYPE_D },
    { 0x189E, UC_JOINING_TYPE_D },
    { 0x189F, UC_JOINING_TYPE_D },
    { 0x18A0, UC_JOINING_TYPE_D },
    { 0x18A1, UC_JOINING_TYPE_D },
    { 0x18A2, UC_JOINING_TYPE_D },
    { 0x18A3, UC_JOINING_TYPE_D },
    { 0x18A4, UC_JOINING_TYPE_D },
    { 0x18A5, UC_JOINING_TYPE_D },
    { 0x18A6, UC_JOINING_TYPE_D },
    { 0x18A7, UC_JOINING_TYPE_D },
    { 0x18A8, UC_JOINING_TYPE_D },
    { 0x18AA, UC_JOINING_TYPE_D },
    { 0x200C, UC_JOINING_TYPE_U },
    { 0x200D, UC_JOINING_TYPE_C },
    { 0x202F, UC_JOINING_TYPE_U },
    { 0x2066, UC_JOINING_TYPE_U },
    { 0x2067, UC_JOINING_TYPE_U },
    { 0x2068, UC_JOINING_TYPE_U },
    { 0x2069, UC_JOINING_TYPE_U },
    { 0xA840, UC_JOINING_TYPE_D },
    { 0xA841, UC_JOINING_TYPE_D },
    { 0xA842, UC_JOINING_TYPE_D },
    { 0xA843, UC_JOINING_TYPE_D },
    { 0xA844, UC_JOINING_TYPE_D },
    { 0xA845, UC_JOINING_TYPE_D },
    { 0xA846, UC_JOINING_TYPE_D },
    { 0xA847, UC_JOINING_TYPE_D },
    { 0xA848, UC_JOINING_TYPE_D },
    { 0xA849, UC_JOINING_TYPE_D },
    { 0xA84A, UC_JOINING_TYPE_D },
    { 0xA84B, UC_JOINING_TYPE_D },
    { 0xA84C, UC_JOINING_TYPE_D },
    { 0xA84D, UC_JOINING_TYPE_D },
    { 0xA84E, UC_JOINING_TYPE_D },
    { 0xA84F, UC_JOINING_TYPE_D },
    { 0xA850, UC_JOINING_TYPE_D },
    { 0xA851, UC_JOINING_TYPE_D },
    { 0xA852, UC_JOINING_TYPE_D },
    { 0xA853, UC_JOINING_TYPE_D },
    { 0xA854, UC_JOINING_TYPE_D },
    { 0xA855, UC_JOINING_TYPE_D },
    { 0xA856, UC_JOINING_TYPE_D },
    { 0xA857, UC_JOINING_TYPE_D },
    { 0xA858, UC_JOINING_TYPE_D },
    { 0xA859, UC_JOINING_TYPE_D },
    { 0xA85A, UC_JOINING_TYPE_D },
    { 0xA85B, UC_JOINING_TYPE_D },
    { 0xA85C, UC_JOINING_TYPE_D },
    { 0xA85D, UC_JOINING_TYPE_D },
    { 0xA85E, UC_JOINING_TYPE_D },
    { 0xA85F, UC_JOINING_TYPE_D },
    { 0xA860, UC_JOINING_TYPE_D },
    { 0xA861, UC_JOINING_TYPE_D },
    { 0xA862, UC_JOINING_TYPE_D },
    { 0xA863, UC_JOINING_TYPE_D },
    { 0xA864, UC_JOINING_TYPE_D },
    { 0xA865, UC_JOINING_TYPE_D },
    { 0xA866, UC_JOINING_TYPE_D },
    { 0xA867, UC_JOINING_TYPE_D },
    { 0xA868, UC_JOINING_TYPE_D },
    { 0xA869, UC_JOINING_TYPE_D },
    { 0xA86A, UC_JOINING_TYPE_D },
    { 0xA86B, UC_JOINING_TYPE_D },
    { 0xA86C, UC_JOINING_TYPE_D },
    { 0xA86D, UC_JOINING_TYPE_D },
    { 0xA86E, UC_JOINING_TYPE_D },
    { 0xA86F, UC_JOINING_TYPE_D },
    { 0xA870, UC_JOINING_TYPE_D },
    { 0xA871, UC_JOINING_TYPE_D },
    { 0xA872, UC_JOINING_TYPE_L },
    { 0xA873, UC_JOINING_TYPE_U },
    { 0x10AC0, UC_JOINING_TYPE_D },
    { 0x10AC1, UC_JOINING_TYPE_D },
    { 0x10AC2, UC_JOINING_TYPE_D },
    { 0x10AC3, UC_JOINING_TYPE_D },
    { 0x10AC4, UC_JOINING_TYPE_D },
    { 0x10AC5, UC_JOINING_TYPE_R },
    { 0x10AC6, UC_JOINING_TYPE_U },
    { 0x10AC7, UC_JOINING_TYPE_R },
    { 0x10AC8, UC_JOINING_TYPE_U },
    { 0x10AC9, UC_JOINING_TYPE_R },
    { 0x10ACA, UC_JOINING_TYPE_R },
    { 0x10ACB, UC_JOINING_TYPE_U },
    { 0x10ACC, UC_JOINING_TYPE_U },
    { 0x10ACD, UC_JOINING_TYPE_L },
    { 0x10ACE, UC_JOINING_TYPE_R },
    { 0x10ACF, UC_JOINING_TYPE_R },
    { 0x10AD0, UC_JOINING_TYPE_R },
    { 0x10AD1, UC_JOINING_TYPE_R },
    { 0x10AD2, UC_JOINING_TYPE_R },
    { 0x10AD3, UC_JOINING_TYPE_D },
    { 0x10AD4, UC_JOINING_TYPE_D },
    { 0x10AD5, UC_JOINING_TYPE_D },
    { 0x10AD6, UC_JOINING_TYPE_D },
    { 0x10AD7, UC_JOINING_TYPE_L },
    { 0x10AD8, UC_JOINING_TYPE_D },
    { 0x10AD9, UC_JOINING_TYPE_D },
    { 0x10ADA, UC_JOINING_TYPE_D },
    { 0x10ADB, UC_JOINING_TYPE_D },
    { 0x10ADC, UC_JOINING_TYPE_D },
    { 0x10ADD, UC_JOINING_TYPE_R },
    { 0x10ADE, UC_JOINING_TYPE_D },
    { 0x10ADF, UC_JOINING_TYPE_D },
    { 0x10AE0, UC_JOINING_TYPE_D },
    { 0x10AE1, UC_JOINING_TYPE_R },
    { 0x10AE2, UC_JOINING_TYPE_U },
    { 0x10AE3, UC_JOINING_TYPE_U },
    { 0x10AE4, UC_JOINING_TYPE_R },
    { 0x10AEB, UC_JOINING_TYPE_D },
    { 0x10AEC, UC_JOINING_TYPE_D },
    { 0x10AED, UC_JOINING_TYPE_D },
    { 0x10AEE, UC_JOINING_TYPE_D },
    { 0x10AEF, UC_JOINING_TYPE_R },
    { 0x10B80, UC_JOINING_TYPE_D },
    { 0x10B81, UC_JOINING_TYPE_R },
    { 0x10B82, UC_JOINING_TYPE_D },
    { 0x10B83, UC_JOINING_TYPE_R },
    { 0x10B84, UC_JOINING_TYPE_R },
    { 0x10B85, UC_JOINING_TYPE_R },
    { 0x10B86, UC_JOINING_TYPE_D },
    { 0x10B87, UC_JOINING_TYPE_D },
    { 0x10B88, UC_JOINING_TYPE_D },
    { 0x10B89, UC_JOINING_TYPE_R },
    { 0x10B8A, UC_JOINING_TYPE_D },
    { 0x10B8B, UC_JOINING_TYPE_D },
    { 0x10B8C, UC_JOINING_TYPE_R },
    { 0x10B8D, UC_JOINING_TYPE_D },
    { 0x10B8E, UC_JOINING_TYPE_R },
    { 0x10B8F, UC_JOINING_TYPE_R },
    { 0x10B90, UC_JOINING_TYPE_D },
    { 0x10B91, UC_JOINING_TYPE_R },
    { 0x10BA9, UC_JOINING_TYPE_R },
    { 0x10BAA, UC_JOINING_TYPE_R },
    { 0x10BAB, UC_JOINING_TYPE_R },
    { 0x10BAC, UC_JOINING_TYPE_R },
    { 0x10BAD, UC_JOINING_TYPE_D },
    { 0x10BAE, UC_JOINING_TYPE_D },
    { 0x10BAF, UC_JOINING_TYPE_U },
    { 0x10D00, UC_JOINING_TYPE_L },
    { 0x10D01, UC_JOINING_TYPE_D },
    { 0x10D02, UC_JOINING_TYPE_D },
    { 0x10D03, UC_JOINING_TYPE_D },
    { 0x10D04, UC_JOINING_TYPE_D },
    { 0x10D05, UC_JOINING_TYPE_D },
    { 0x10D06, UC_JOINING_TYPE_D },
    { 0x10D07, UC_JOINING_TYPE_D },
    { 0x10D08, UC_JOINING_TYPE_D },
    { 0x10D09, UC_JOINING_TYPE_D },
    { 0x10D0A, UC_JOINING_TYPE_D },
    { 0x10D0B, UC_JOINING_TYPE_D },
    { 0x10D0C, UC_JOINING_TYPE_D },
    { 0x10D0D, UC_JOINING_TYPE_D },
    { 0x10D0E, UC_JOINING_TYPE_D },
    { 0x10D0F, UC_JOINING_TYPE_D },
    { 0x10D10, UC_JOINING_TYPE_D },
    { 0x10D11, UC_JOINING_TYPE_D },
    { 0x10D12, UC_JOINING_TYPE_D },
    { 0x10D13, UC_JOINING_TYPE_D },
    { 0x10D14, UC_JOINING_TYPE_D },
    { 0x10D15, UC_JOINING_TYPE_D },
    { 0x10D16, UC_JOINING_TYPE_D },
    { 0x10D17, UC_JOINING_TYPE_D },
    { 0x10D18, UC_JOINING_TYPE_D },
    { 0x10D19, UC_JOINING_TYPE_D },
    { 0x10D1A, UC_JOINING_TYPE_D },
    { 0x10D1B, UC_JOINING_TYPE_D },
    { 0x10D1C, UC_JOINING_TYPE_D },
    { 0x10D1D, UC_JOINING_TYPE_D },
    { 0x10D1E, UC_JOINING_TYPE_D },
    { 0x10D1F, UC_JOINING_TYPE_D },
    { 0x10D20, UC_JOINING_TYPE_D },
    { 0x10D21, UC_JOINING_TYPE_D },
    { 0x10D22, UC_JOINING_TYPE_R },
    { 0x10D23, UC_JOINING_TYPE_D },
    { 0x10EC2, UC_JOINING_TYPE_R },
    { 0x10EC3, UC_JOINING_TYPE_D },
    { 0x10EC4, UC_JOINING_TYPE_D },
    { 0x10EC6, UC_JOINING_TYPE_D },
    { 0x10EC7, UC_JOINING_TYPE_D },
    { 0x10F30, UC_JOINING_TYPE_D },
    { 0x10F31, UC_JOINING_TYPE_D },
    { 0x10F32, UC_JOINING_TYPE_D },
    { 0x10F33, UC_JOINING_TYPE_R },
    { 0x10F34, UC_JOINING_TYPE_D },
    { 0x10F35, UC_JOINING_TYPE_D },
    { 0x10F36, UC_JOINING_TYPE_D },
    { 0x10F37, UC_JOINING_TYPE_D },
    { 0x10F38, UC_JOINING_TYPE_D },
    { 0x10F39, UC_JOINING_TYPE_D },
    { 0x10F3A, UC_JOINING_TYPE_D },
    { 0x10F3B, UC_JOINING_TYPE_D },
    { 0x10F3C, UC_JOINING_TYPE_D },
    { 0x10F3D, UC_JOINING_TYPE_D },
    { 0x10F3E, UC_JOINING_TYPE_D },
    { 0x10F3F, UC_JOINING_TYPE_D },
    { 0x10F40, UC_JOINING_TYPE_D },
    { 0x10F41, UC_JOINING_TYPE_D },
    { 0x10F42, UC_JOINING_TYPE_D },
    { 0x10F43, UC_JOINING_TYPE_D },
    { 0x10F44, UC_JOINING_TYPE_D },
    { 0x10F45, UC_JOINING_TYPE_U },
    { 0x10F51, UC_JOINING_TYPE_D },
    { 0x10F52, UC_JOINING_TYPE_D },
    { 0x10F53, UC_JOINING_TYPE_D },
    { 0x10F54, UC_JOINING_TYPE_R },
    { 0x10F70, UC_JOINING_TYPE_D },
    { 0x10F71, UC_JOINING_TYPE_D },
    { 0x10F72, UC_JOINING_TYPE_D },
    { 0x10F73, UC_JOINING_TYPE_D },
    { 0x10F74, UC_JOINING_TYPE_R },
    { 0x10F75, UC_JOINING_TYPE_R },
    { 0x10F76, UC_JOINING_TYPE_D },
    { 0x10F77, UC_JOINING_TYPE_D },
    { 0x10F78, UC_JOINING_TYPE_D },
    { 0x10F79, UC_JOINING_TYPE_D },
    { 0x10F7A, UC_JOINING_TYPE_D },
    { 0x10F7B, UC_JOINING_TYPE_D },
    { 0x10F7C, UC_JOINING_TYPE_D },
    { 0x10F7D, UC_JOINING_TYPE_D },
    { 0x10F7E, UC_JOINING_TYPE_D },
    { 0x10F7F, UC_JOINING_TYPE_D },
    { 0x10F80, UC_JOINING_TYPE_D },
    { 0x10F81, UC_JOINING_TYPE_D },
    { 0x10FB0, UC_JOINING_TYPE_D },
    { 0x10FB1, UC_JOINING_TYPE_U },
    { 0x10FB2, UC_JOINING_TYPE_D },
    { 0x10FB3, UC_JOINING_TYPE_D },
    { 0x10FB4, UC_JOINING_TYPE_R },
    { 0x10FB5, UC_JOINING_TYPE_R },
    { 0x10FB6, UC_JOINING_TYPE_R },
    { 0x10FB7, UC_JOINING_TYPE_U },
    { 0x10FB8, UC_JOINING_TYPE_D },
    { 0x10FB9, UC_JOINING_TYPE_R },
    { 0x10FBA, UC_JOINING_TYPE_R },
    { 0x10FBB, UC_JOINING_TYPE_D },
    { 0x10FBC, UC_JOINING_TYPE_D },
    { 0x10FBD, UC_JOINING_TYPE_R },
    { 0x10FBE, UC_JOINING_TYPE_D },
    { 0x10FBF, UC_JOINING_TYPE_D },
    { 0x10FC0, UC_JOINING_TYPE_U },
    { 0x10FC1, UC_JOINING_TYPE_D },
    { 0x10FC2, UC_JOINING_TYPE_R },
    { 0x10FC3, UC_JOINING_TYPE_R },
    { 0x10FC4, UC_JOINING_TYPE_D },
    { 0x10FC5, UC_JOINING_TYPE_U },
    { 0x10FC6, UC_JOINING_TYPE_U },
    { 0x10FC7, UC_JOINING_TYPE_U },
    { 0x10FC8, UC_JOINING_TYPE_U },
    { 0x10FC9, UC_JOINING_TYPE_R },
    { 0x10FCA, UC_JOINING_TYPE_D },
    { 0x10FCB, UC_JOINING_TYPE_L },
    { 0x110BD, UC_JOINING_TYPE_U },
    { 0x110CD, UC_JOINING_TYPE_U },
    { 0x1E900, UC_JOINING_TYPE_D },
    { 0x1E901, UC_JOINING_TYPE_D },
    { 0x1E902, UC_JOINING_TYPE_D },
    { 0x1E903, UC_JOINING_TYPE_D },
    { 0x1E904, UC_JOINING_TYPE_D },
    { 0x1E905, UC_JOINING_TYPE_D },
    { 0x1E906, UC_JOINING_TYPE_D },
    { 0x1E907, UC_JOINING_TYPE_D },
    { 0x1E908, UC_JOINING_TYPE_D },
    { 0x1E909, UC_JOINING_TYPE_D },
    { 0x1E90A, UC_JOINING_TYPE_D },
    { 0x1E90B, UC_JOINING_TYPE_D },
    { 0x1E90C, UC_JOINING_TYPE_D },
    { 0x1E90D, UC_JOINING_TYPE_D },
    { 0x1E90E, UC_JOINING_TYPE_D },
    { 0x1E90F, UC_JOINING_TYPE_D },
    { 0x1E910, UC_JOINING_TYPE_D },
    { 0x1E911, UC_JOINING_TYPE_D },
    { 0x1E912, UC_JOINING_TYPE_D },
    { 0x1E913, UC_JOINING_TYPE_D },
    { 0x1E914, UC_JOINING_TYPE_D },
    { 0x1E915, UC_JOINING_TYPE_D },
    { 0x1E916, UC_JOINING_TYPE_D },
    { 0x1E917, UC_JOINING_TYPE_D },
    { 0x1E918, UC_JOINING_TYPE_D },
    { 0x1E919, UC_JOINING_TYPE_D },
    { 0x1E91A, UC_JOINING_TYPE_D },
    { 0x1E91B, UC_JOINING_TYPE_D },
    { 0x1E91C, UC_JOINING_TYPE_D },
    { 0x1E91D, UC_JOINING_TYPE_D },
    { 0x1E91E, UC_JOINING_TYPE_D },
    { 0x1E91F, UC_JOINING_TYPE_D },
    { 0x1E920, UC_JOINING_TYPE_D },
    { 0x1E921, UC_JOINING_TYPE_D },
    { 0x1E922, UC_JOINING_TYPE_D },
    { 0x1E923, UC_JOINING_TYPE_D },
    { 0x1E924, UC_JOINING_TYPE_D },
    { 0x1E925, UC_JOINING_TYPE_D },
    { 0x1E926, UC_JOINING_TYPE_D },
    { 0x1E927, UC_JOINING_TYPE_D },
    { 0x1E928, UC_JOINING_TYPE_D },
    { 0x1E929, UC_JOINING_TYPE_D },
    { 0x1E92A, UC_JOINING_TYPE_D },
    { 0x1E92B, UC_JOINING_TYPE_D },
    { 0x1E92C, UC_JOINING_TYPE_D },
    { 0x1E92D, UC_JOINING_TYPE_D },
    { 0x1E92E, UC_JOINING_TYPE_D },
    { 0x1E92F, UC_JOINING_TYPE_D },
    { 0x1E930, UC_JOINING_TYPE_D },
    { 0x1E931, UC_JOINING_TYPE_D },
    { 0x1E932, UC_JOINING_TYPE_D },
    { 0x1E933, UC_JOINING_TYPE_D },
    { 0x1E934, UC_JOINING_TYPE_D },
    { 0x1E935, UC_JOINING_TYPE_D },
    { 0x1E936, UC_JOINING_TYPE_D },
    { 0x1E937, UC_JOINING_TYPE_D },
    { 0x1E938, UC_JOINING_TYPE_D },
    { 0x1E939, UC_JOINING_TYPE_D },
    { 0x1E93A, UC_JOINING_TYPE_D },
    { 0x1E93B, UC_JOINING_TYPE_D },
    { 0x1E93C, UC_JOINING_TYPE_D },
    { 0x1E93D, UC_JOINING_TYPE_D },
    { 0x1E93E, UC_JOINING_TYPE_D },
    { 0x1E93F, UC_JOINING_TYPE_D },
    { 0x1E940, UC_JOINING_TYPE_D },
    { 0x1E941, UC_JOINING_TYPE_D },
    { 0x1E942, UC_JOINING_TYPE_D },
    { 0x1E943, UC_JOINING_TYPE_D },
    { 0x1E94B, UC_JOINING_TYPE_T }
