/* DO NOT EDIT! GENERATED AUTOMATICALLY! */
/* Test the Unicode character type functions.
   Copyright (C) 2007-2025 Free Software Foundation, Inc.

   This file is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published
   by the Free Software Foundation, either version 3 of the License,
   or (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */

#include "test-predicate-part1.h"

    { 0x0069, 0x006A },
    { 0x012F, 0x012F },
    { 0x0249, 0x0249 },
    { 0x0268, 0x0268 },
    { 0x029D, 0x029D },
    { 0x02B2, 0x02B2 },
    { 0x03F3, 0x03F3 },
    { 0x0456, 0x0456 },
    { 0x0458, 0x0458 },
    { 0x1D62, 0x1D62 },
    { 0x1D96, 0x1D96 },
    { 0x1DA4, 0x1DA4 },
    { 0x1DA8, 0x1DA8 },
    { 0x1E2D, 0x1E2D },
    { 0x1ECB, 0x1ECB },
    { 0x2071, 0x2071 },
    { 0x2148, 0x2149 },
    { 0x2C7C, 0x2C7C },
    { 0x1D422, 0x1D423 },
    { 0x1D456, 0x1D457 },
    { 0x1D48A, 0x1D48B },
    { 0x1D4BE, 0x1D4BF },
    { 0x1D4F2, 0x1D4F3 },
    { 0x1D526, 0x1D527 },
    { 0x1D55A, 0x1D55B },
    { 0x1D58E, 0x1D58F },
    { 0x1D5C2, 0x1D5C3 },
    { 0x1D5F6, 0x1D5F7 },
    { 0x1D62A, 0x1D62B },
    { 0x1D65E, 0x1D65F },
    { 0x1D692, 0x1D693 },
    { 0x1DF1A, 0x1DF1A },
    { 0x1E04C, 0x1E04D },
    { 0x1E068, 0x1E068 }

#define PREDICATE(c) uc_is_property_soft_dotted (c)
#include "test-predicate-part2.h"
