/* DO NOT EDIT! GENERATED AUTOMATICALLY! */
/* Test the Unicode character type functions.
   Copyright (C) 2007-2025 Free Software Foundation, Inc.

   This file is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published
   by the Free Software Foundation, either version 3 of the License,
   or (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */

#include "test-predicate-part1.h"

    { 0x09BE, 0x09BE },
    { 0x09D7, 0x09D7 },
    { 0x0B3E, 0x0B3E },
    { 0x0B57, 0x0B57 },
    { 0x0BBE, 0x0BBE },
    { 0x0BD7, 0x0BD7 },
    { 0x0CC0, 0x0CC0 },
    { 0x0CC2, 0x0CC2 },
    { 0x0CC7, 0x0CC8 },
    { 0x0CCA, 0x0CCB },
    { 0x0CD5, 0x0CD6 },
    { 0x0D3E, 0x0D3E },
    { 0x0D57, 0x0D57 },
    { 0x0DCF, 0x0DCF },
    { 0x0DDF, 0x0DDF },
    { 0x1715, 0x1715 },
    { 0x1734, 0x1734 },
    { 0x1B35, 0x1B35 },
    { 0x1B3B, 0x1B3B },
    { 0x1B3D, 0x1B3D },
    { 0x1B43, 0x1B44 },
    { 0x1BAA, 0x1BAA },
    { 0x1BF2, 0x1BF3 },
    { 0x200C, 0x200C },
    { 0x302E, 0x302F },
    { 0xA953, 0xA953 },
    { 0xA9C0, 0xA9C0 },
    { 0xFF9E, 0xFF9F },
    { 0x111C0, 0x111C0 },
    { 0x11235, 0x11235 },
    { 0x1133E, 0x1133E },
    { 0x1134D, 0x1134D },
    { 0x11357, 0x11357 },
    { 0x113B8, 0x113B8 },
    { 0x113C2, 0x113C2 },
    { 0x113C5, 0x113C5 },
    { 0x113C7, 0x113C9 },
    { 0x113CF, 0x113CF },
    { 0x114B0, 0x114B0 },
    { 0x114BD, 0x114BD },
    { 0x115AF, 0x115AF },
    { 0x116B6, 0x116B6 },
    { 0x11930, 0x11930 },
    { 0x1193D, 0x1193D },
    { 0x11F41, 0x11F41 },
    { 0x16FF0, 0x16FF1 },
    { 0x1D165, 0x1D166 },
    { 0x1D16D, 0x1D172 },
    { 0xE0020, 0xE007F }

#define PREDICATE(c) uc_is_property_other_grapheme_extend (c)
#include "test-predicate-part2.h"
