/* Search wide string for a wide character.
   Copyright (C) 1999, 2011-2026 Free Software Foundation, Inc.
   Written by Bruno Haible <bruno@clisp.org>, 1999.

   This file is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of the
   License, or (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */

wchar_t *
wcschr (const wchar_t *wcs, wchar_t wc)
{
  for (;; wcs++)
    {
      if (*wcs == wc)
        break;
      if (*wcs == (wchar_t)'\0')
        goto notfound;
    }
  return (wchar_t *) wcs;

 notfound:
  return NULL;
}
