/* DO NOT EDIT! GENERATED AUTOMATICALLY! */
/* Test the Unicode character type functions.
   Copyright (C) 2007-2025 Free Software Foundation, Inc.

   This file is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published
   by the Free Software Foundation, either version 3 of the License,
   or (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */

#include "test-predicate-part1.h"

    { 0x261D, 0x261D },
    { 0x26F9, 0x26F9 },
    { 0x270A, 0x270D },
    { 0x1F385, 0x1F385 },
    { 0x1F3C2, 0x1F3C4 },
    { 0x1F3C7, 0x1F3C7 },
    { 0x1F3CA, 0x1F3CC },
    { 0x1F442, 0x1F443 },
    { 0x1F446, 0x1F450 },
    { 0x1F466, 0x1F478 },
    { 0x1F47C, 0x1F47C },
    { 0x1F481, 0x1F483 },
    { 0x1F485, 0x1F487 },
    { 0x1F48F, 0x1F48F },
    { 0x1F491, 0x1F491 },
    { 0x1F4AA, 0x1F4AA },
    { 0x1F574, 0x1F575 },
    { 0x1F57A, 0x1F57A },
    { 0x1F590, 0x1F590 },
    { 0x1F595, 0x1F596 },
    { 0x1F645, 0x1F647 },
    { 0x1F64B, 0x1F64F },
    { 0x1F6A3, 0x1F6A3 },
    { 0x1F6B4, 0x1F6B6 },
    { 0x1F6C0, 0x1F6C0 },
    { 0x1F6CC, 0x1F6CC },
    { 0x1F90C, 0x1F90C },
    { 0x1F90F, 0x1F90F },
    { 0x1F918, 0x1F91F },
    { 0x1F926, 0x1F926 },
    { 0x1F930, 0x1F939 },
    { 0x1F93C, 0x1F93E },
    { 0x1F977, 0x1F977 },
    { 0x1F9B5, 0x1F9B6 },
    { 0x1F9B8, 0x1F9B9 },
    { 0x1F9BB, 0x1F9BB },
    { 0x1F9CD, 0x1F9CF },
    { 0x1F9D1, 0x1F9DD },
    { 0x1FAC3, 0x1FAC5 },
    { 0x1FAF0, 0x1FAF8 }

#define PREDICATE(c) uc_is_property_emoji_modifier_base (c)
#include "test-predicate-part2.h"
