/* DO NOT EDIT! GENERATED AUTOMATICALLY! */
/* Test the Unicode character type functions.
   Copyright (C) 2007-2025 Free Software Foundation, Inc.

   This file is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published
   by the Free Software Foundation, either version 3 of the License,
   or (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */

#include "test-predicate-part1.h"

    { 0x094D, 0x094D },
    { 0x09CD, 0x09CD },
    { 0x0A4D, 0x0A4D },
    { 0x0ACD, 0x0ACD },
    { 0x0B4D, 0x0B4D },
    { 0x0BCD, 0x0BCD },
    { 0x0C4D, 0x0C4D },
    { 0x0CCD, 0x0CCD },
    { 0x0D3B, 0x0D3C },
    { 0x0D4D, 0x0D4D },
    { 0x0DCA, 0x0DCA },
    { 0x0E3A, 0x0E3A },
    { 0x0EBA, 0x0EBA },
    { 0x0F84, 0x0F84 },
    { 0x1039, 0x103A },
    { 0x1714, 0x1715 },
    { 0x1734, 0x1734 },
    { 0x17D2, 0x17D2 },
    { 0x1A60, 0x1A60 },
    { 0x1B44, 0x1B44 },
    { 0x1BAA, 0x1BAB },
    { 0x1BF2, 0x1BF3 },
    { 0x2D7F, 0x2D7F },
    { 0xA806, 0xA806 },
    { 0xA82C, 0xA82C },
    { 0xA8C4, 0xA8C4 },
    { 0xA953, 0xA953 },
    { 0xA9C0, 0xA9C0 },
    { 0xAAF6, 0xAAF6 },
    { 0xABED, 0xABED },
    { 0x10A3F, 0x10A3F },
    { 0x11046, 0x11046 },
    { 0x11070, 0x11070 },
    { 0x1107F, 0x1107F },
    { 0x110B9, 0x110B9 },
    { 0x11133, 0x11134 },
    { 0x111C0, 0x111C0 },
    { 0x11235, 0x11235 },
    { 0x112EA, 0x112EA },
    { 0x1134D, 0x1134D },
    { 0x113CE, 0x113D0 },
    { 0x11442, 0x11442 },
    { 0x114C2, 0x114C2 },
    { 0x115BF, 0x115BF },
    { 0x1163F, 0x1163F },
    { 0x116B6, 0x116B6 },
    { 0x1172B, 0x1172B },
    { 0x11839, 0x11839 },
    { 0x1193D, 0x1193E },
    { 0x119E0, 0x119E0 },
    { 0x11A34, 0x11A34 },
    { 0x11A47, 0x11A47 },
    { 0x11A99, 0x11A99 },
    { 0x11C3F, 0x11C3F },
    { 0x11D44, 0x11D45 },
    { 0x11D97, 0x11D97 },
    { 0x11F41, 0x11F42 },
    { 0x1612F, 0x1612F }

#define PREDICATE(c) uc_is_property_grapheme_link (c)
#include "test-predicate-part2.h"
