/* DO NOT EDIT! GENERATED AUTOMATICALLY! */
/* Properties of Unicode characters.  */
/* Generated automatically by gen-uni-tables.c for Unicode 17.0.0.  */

/* Copyright (C) 2000-2025 Free Software Foundation, Inc.

   This file is free software.
   It is dual-licensed under "the GNU LGPLv3+ or the GNU GPLv2+".
   You can redistribute it and/or modify it under either
     - the terms of the GNU Lesser General Public License as published
       by the Free Software Foundation, either version 3, or (at your
       option) any later version, or
     - the terms of the GNU General Public License as published by the
       Free Software Foundation; either version 2, or (at your option)
       any later version, or
     - the same dual license "the GNU LGPLv3+ or the GNU GPLv2+".

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License and the GNU General Public License
   for more details.

   You should have received a copy of the GNU Lesser General Public
   License and of the GNU General Public License along with this
   program.  If not, see <https://www.gnu.org/licenses/>.  */

#define header_0 16
#define header_2 9
#define header_3 127
#define header_4 15
static const
struct
  {
    int header[1];
    int level1[17];
    short level2[4 << 7];
    unsigned int level3[67 << 4];
  }
u_property_bidi_left_to_right =
{
  { 17 },
  {
       18 * sizeof (int) / sizeof (short) +     0,
       18 * sizeof (int) / sizeof (short) +   128,
       18 * sizeof (int) / sizeof (short) +   256,
       18 * sizeof (int) / sizeof (short) +   256,
       18 * sizeof (int) / sizeof (short) +   256,
       18 * sizeof (int) / sizeof (short) +   256,
       18 * sizeof (int) / sizeof (short) +   256,
       18 * sizeof (int) / sizeof (short) +   256,
       18 * sizeof (int) / sizeof (short) +   256,
       18 * sizeof (int) / sizeof (short) +   256,
       18 * sizeof (int) / sizeof (short) +   256,
       18 * sizeof (int) / sizeof (short) +   256,
       18 * sizeof (int) / sizeof (short) +   256,
       18 * sizeof (int) / sizeof (short) +   256,
       18 * sizeof (int) / sizeof (short) +   384,
       18 * sizeof (int) / sizeof (short) +   256,
       18 * sizeof (int) / sizeof (short) +   256
  },
  {
       18 +   512 * sizeof (short) / sizeof (int) +     0,
       18 +   512 * sizeof (short) / sizeof (int) +    16,
       18 +   512 * sizeof (short) / sizeof (int) +    32,
       -1,
       18 +   512 * sizeof (short) / sizeof (int) +    48,
       18 +   512 * sizeof (short) / sizeof (int) +    64,
       18 +   512 * sizeof (short) / sizeof (int) +    80,
       18 +   512 * sizeof (short) / sizeof (int) +    96,
       18 +   512 * sizeof (short) / sizeof (int) +   112,
       18 +   512 * sizeof (short) / sizeof (int) +   128,
       18 +   512 * sizeof (short) / sizeof (int) +   144,
       18 +   512 * sizeof (short) / sizeof (int) +   160,
       18 +   512 * sizeof (short) / sizeof (int) +   176,
       18 +   512 * sizeof (short) / sizeof (int) +   192,
       18 +   512 * sizeof (short) / sizeof (int) +   208,
       18 +   512 * sizeof (short) / sizeof (int) +   224,
       18 +   512 * sizeof (short) / sizeof (int) +   240,
       18 +   512 * sizeof (short) / sizeof (int) +   256,
       18 +   512 * sizeof (short) / sizeof (int) +   272,
       18 +   512 * sizeof (short) / sizeof (int) +   288,
       18 +   512 * sizeof (short) / sizeof (int) +   304,
       18 +   512 * sizeof (short) / sizeof (int) +   320,
       18 +   512 * sizeof (short) / sizeof (int) +   336,
       18 +   512 * sizeof (short) / sizeof (int) +   352,
       18 +   512 * sizeof (short) / sizeof (int) +   368,
       18 +   512 * sizeof (short) / sizeof (int) +   384,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   416,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   432,
       18 +   512 * sizeof (short) / sizeof (int) +   448,
       18 +   512 * sizeof (short) / sizeof (int) +   464,
       18 +   512 * sizeof (short) / sizeof (int) +   480,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   496,
       -1,
       18 +   512 * sizeof (short) / sizeof (int) +   512,
       18 +   512 * sizeof (short) / sizeof (int) +   528,
       18 +   512 * sizeof (short) / sizeof (int) +   544,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       -1,
       -1,
       -1,
       -1,
       18 +   512 * sizeof (short) / sizeof (int) +   560,
       18 +   512 * sizeof (short) / sizeof (int) +   576,
       18 +   512 * sizeof (short) / sizeof (int) +   592,
       18 +   512 * sizeof (short) / sizeof (int) +   608,
       18 +   512 * sizeof (short) / sizeof (int) +   624,
       18 +   512 * sizeof (short) / sizeof (int) +   640,
       18 +   512 * sizeof (short) / sizeof (int) +   656,
       18 +   512 * sizeof (short) / sizeof (int) +   672,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   688,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   704,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   720,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   736,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   752,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   768,
       18 +   512 * sizeof (short) / sizeof (int) +   784,
       18 +   512 * sizeof (short) / sizeof (int) +   800,
       18 +   512 * sizeof (short) / sizeof (int) +   816,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   832,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   848,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   864,
       18 +   512 * sizeof (short) / sizeof (int) +   880,
       18 +   512 * sizeof (short) / sizeof (int) +   896,
       18 +   512 * sizeof (short) / sizeof (int) +   912,
       18 +   512 * sizeof (short) / sizeof (int) +   928,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   944,
       18 +   512 * sizeof (short) / sizeof (int) +   960,
       18 +   512 * sizeof (short) / sizeof (int) +   976,
       18 +   512 * sizeof (short) / sizeof (int) +   992,
       -1,
       18 +   512 * sizeof (short) / sizeof (int) +  1008,
       18 +   512 * sizeof (short) / sizeof (int) +  1024,
       18 +   512 * sizeof (short) / sizeof (int) +  1040,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +  1056,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +  1056,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +   400,
       18 +   512 * sizeof (short) / sizeof (int) +  1056
  },
  {
    0x00000000U, 0x00000000U, 0x07FFFFFEU, 0x07FFFFFEU,
    0x00000000U, 0x04200400U, 0xFF7FFFFFU, 0xFF7FFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xF9FFFFFFU, 0x00030003U, 0x0000401FU,
    0x00000000U, 0x00000000U, 0x00000000U, 0xBFCF0000U,
    0xFFFFFF4FU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFBFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFC07U, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0x00001BFFU, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0xFFFFFFF8U, 0xEBFFFFFFU, 0xFF01DE01U, 0xFFFFFFF3U,
    0xFFFFFFFDU, 0xEFFFFFFFU, 0xFFFFDFE1U, 0xB7F3FFF3U,
    0xFFFFFFF9U, 0xEFFFFFFFU, 0xFFFDC679U, 0xFFDCFFFFU,
    0xFFFFFFF9U, 0xEFFFFFFFU, 0xFFFFDE41U, 0x03FDFFF3U,
    0xFFFFFFFDU, 0x6FFFFFFFU, 0xFF9FDFE1U, 0xFFFFFFF3U,
    0xFFFFFFFBU, 0xFFFFFFFFU, 0xFFFFDFFEU, 0xF807FFFFU,
    0xFFFFFFEEU, 0x2FFFFFFFU, 0xFF9FC23EU, 0x80FFFFF3U,
    0xFFFFFFFDU, 0xEFFFFFFFU, 0xFFFFCFFFU, 0xFFFFFFF3U,
    0xFFFFFFFCU, 0xE7FFFFFFU, 0xFFFFDFE1U, 0xFFFFFFF3U,
    0xFFFFFFFDU, 0xFFFFFFFFU, 0xFFA3FBFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0x780DFFFFU, 0xFFFF807FU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xE00DFFFFU, 0xFFFF80FFU, 0xFFFFFFFFU,
    0xFCFFFFFFU, 0xC15FFFFFU, 0xFFFFFFFFU, 0x8001FFFFU,
    0x01001F20U, 0xE0000000U, 0xFFFFFFBFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0x99021FFFU, 0x3CFFFFFFU, 0xFFE1FFFEU,
    0xDFFFDF9BU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0x1FFFFFFFU, 0xFFFFFFFFU,
    0xFC00FFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFEU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xE7FFFFFEU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFE3FFFFU, 0xFFF3FFFFU, 0xFFF3FFFFU, 0xFFF3FFFFU,
    0xFFFFFFFFU, 0xC04FFFFFU, 0xD7F001BFU, 0xFC00FFFFU,
    0xFFFF0000U, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFF9FU, 0xFFFFFDFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xF1FBFE78U, 0xFFFFFFCEU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0x3FFFFFFFU, 0x00000000U,
    0xF67FFFFFU, 0xFFFFFFFFU, 0x80BFFFFFU, 0x6007E01AU,
    0xFFFFFFFFU, 0x0000FFFFU, 0xC0000000U, 0xFFFFF000U,
    0xFFFFFFF0U, 0xE82FFFFFU, 0xFFFFFFFBU, 0xFFF007FFU,
    0xFFFFFFFCU, 0xFFFFC4C3U, 0xFFFFFFFFU, 0xFFFC5CBFU,
    0xFFFFFFFFU, 0xFF300FFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0x0008FFFFU, 0xFCEFDE02U,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0x00000000U, 0x00000000U,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0x5FFFFFFFU, 0x1FFF1FFCU, 0x9FFF1FFFU,
    0x00004000U, 0x00000000U, 0x00000000U, 0x800E0000U,
    0xFFFF8000U, 0x00000000U, 0x0000FFFCU, 0xFFFE0000U,
    0x3E2FFC84U, 0xF3FFBD50U, 0x0000C3E0U, 0xFFFFFFFFU,
    0x0000F1FFU, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0xFFC00000U, 0xFFFFFFFFU, 0x07FFFFFFU,
    0x00200000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0xFFFFFC00U, 0xFFFFF800U, 0x00000000U,
    0xF0000000U, 0xFFFFFFFFU, 0xFFFFFFFFU, 0x000003FFU,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00001000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00300000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0x01FC781FU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0x7FFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0x00000000U,
    0x00000000U, 0x00000000U, 0xC0000000U, 0xFFFFFFFFU,
    0x04000000U, 0x00000000U, 0x00000000U, 0xFFF00000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0xFFC00000U, 0x0000FFFFU,
    0x000000E0U, 0x1F3EC3FEU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xE1FFFFFFU, 0xFFFFFFFEU, 0xFFFFFFFFU, 0xF7FFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0x00000000U, 0xFFFF7FC0U,
    0x9FFFFFFFU, 0xFFFFFFFFU, 0x0000FFFFU, 0x8FFFFFFFU,
    0xFFFFFFFFU, 0x0001FFFFU, 0xFFFF0FFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xF87FFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0x3FFFFFFFU, 0x7FFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0x00000000U, 0x00000000U,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0x0000FFFFU, 0x00000000U, 0xFFFFFF80U, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFF1FFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0x00007FFFU,
    0x3FFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFCFFFFU,
    0x00000000U, 0xFFFFFFFCU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFEFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFF7BBU, 0xFCFFE09FU, 0xFFFFFFFFU, 0xFF0FFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFCFU, 0x7FFC0000U,
    0xFFFFFFFFU, 0xFFFFC03FU, 0xFFFC007FU, 0xFFFFFFFFU,
    0xFFFFFFF8U, 0xCC37FFFFU, 0xFFFFFFFFU, 0xFFFFFFDFU,
    0xFFFFFFFFU, 0xFF9981FFU, 0xFFFFEFF7U, 0xEFFFFFFFU,
    0xFFFFFFFFU, 0x3E62FFFFU, 0xFFFFFFFDU, 0xFFBFCFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFF3FFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFDEDFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0x1FFFFFFFU, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000001U, 0x07FFFFFEU, 0x07FFFFFEU, 0xFFFFFFC0U,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0x00008080U,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFDU, 0xFFFFFFFFU, 0x00000000U, 0x00000000U,
    0xE000E000U, 0xFFFFFFFEU, 0xFFFFFFFFU, 0xDFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xF0000000U,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xF83FFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFDU, 0x00FFFFFFU, 0x0003FF80U, 0x7FE6FFC0U,
    0xFFFFFFFCU, 0xF987FFFFU, 0xFFFFFFFBU, 0xFFFFFFFFU,
    0xFFFFFFF8U, 0xFFE0107FU, 0xFFFFFFFFU, 0xFFF7FFFFU,
    0xFFFFFFFCU, 0x803FFFFFU, 0xFFFF61FFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xBF2C7FFFU, 0xFFFFFFFDU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0x7FFFFFFFU, 0xFFFFF807U,
    0xFFFFFFFCU, 0xE7FFFFFFU, 0xFFFFFFFEU, 0xFFE0E03FU,
    0xFFFFFFFFU, 0x07FFFFFFU, 0xFFFABFFEU, 0xFFFFFFF9U,
    0xFFFFFFFFU, 0x00FFFFFFU, 0xBFFFFFA3U, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0x7A07FFFFU, 0xFFFFFFF2U, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0x4FC3FFFFU, 0xCFFFFFFEU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0x5807FFFFU, 0xFFFFFFFEU, 0xFFFFE000U,
    0xFFFFFFFFU, 0xFF40D7FFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0x5FFFFFFFU, 0xFFFFF043U, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xF9007FFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xA7FFFFFFU, 0xFFFFFFF7U, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xF30FFFFFU, 0xFFFFFFFEU,
    0xFFFFF981U, 0x8607FFFFU, 0xF181FF7FU, 0xFFFFFFFFU,
    0xFC8003FFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFA2U,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xC080FFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0x0003FFFFU, 0xFF920300U, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0x4B81FFFFU, 0xFFFFFF40U, 0xFFFFFFFFU,
    0xFF5CFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFE7FFFFU,
    0xFFFFFFFCU, 0xF83FFFFFU, 0xFBFFFFFAU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0x001FFFFFU, 0xFFFC0000U,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFC0007EU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0x3FFFFFFFU, 0xFFFF1C00U, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFE0FFFFU,
    0xFFFFFFFFU, 0xFF80FFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFF7FFFU, 0xFFFFFFFFU,
    0xFFF87FFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFEBU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0x9FFFFFFFU, 0xFFFFFFF0U, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0xFFC00000U, 0xE000FFFFU,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x03F00000U, 0xFFFE0000U, 0xFFFE0000U,
    0x00000000U, 0x0000C000U, 0xFFFFFF80U, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0x0007FC7FU,
    0xFFFFF018U, 0xFFFFC3FFU, 0xFFFFFFFFU, 0xFFFFF9FFU,
    0x00000000U, 0x00000000U, 0xFFFFFFC0U, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0x00000000U, 0x00000000U, 0xFF800000U, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xF7FFFFFDU, 0xF7FFFFFFU,
    0xFFDFFFFFU, 0xFFDFFFFFU, 0xFFFF7FFFU, 0xFFFF7FFFU,
    0xFFFFFDFFU, 0xFFFFFDFFU, 0x00003FF7U, 0x00000000U,
    0x00000000U, 0x07800000U, 0x00000000U, 0xFFDFE000U,
    0x07FFFFEFU, 0xFFFF0001U, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0x06000080U, 0xFFFFF824U, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFF7FFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFF80FFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFBFFFU, 0xFFFFFFFFU, 0x7FFF0FFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFF0FFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFF3FFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFDF3FB7U,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0xFF800060U, 0xFFFFFFFFU,
    0x00000000U, 0x00000000U, 0x3C00F000U, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0x0001FFFFU,
    0x00000000U, 0xFFE00000U, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0x00000001U, 0xC0000000U, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0x00000010U, 0xF5080169U, 0x5569157BU, 0xA1080869U,
    0xF0000400U, 0xF0000411U, 0xFFFFFFFFU, 0xFFFCFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0x00000000U, 0x0000F000U, 0x00000000U, 0x00000000U,
    0xFFF00000U, 0x00018000U, 0x00010001U, 0xFFC00000U,
    0xFFFF0000U, 0xFFFF7FFFU, 0xFFFFFFFFU, 0xFFFF03FFU,
    0xFFFFFFFFU, 0xFFFFDFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFC0U,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x0E000000U, 0xE000E000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0xFC000000U, 0xFFFEF000U,
    0x0000F000U, 0x00000000U, 0xFC00FF00U, 0x00000000U,
    0x0000FF00U, 0xF000C000U, 0xFE00FFFCU, 0xFFFFFFFFU,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00000000U, 0x00000000U, 0xFF000000U, 0xE000C000U,
    0x00003800U, 0x00000000U, 0x60001E80U, 0xFE007800U,
    0x00000000U, 0x00000000U, 0x00000000U, 0x00000000U,
    0x00080000U, 0x00000000U, 0x00000000U, 0xF8000000U,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xFFFFFFFFU, 0xFFFFFFFFU, 0x3FFFFFFFU
  }
};
