/* DO NOT EDIT! GENERATED AUTOMATICALLY! */
/* Test the Unicode character type functions.
   Copyright (C) 2007-2025 Free Software Foundation, Inc.

   This file is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published
   by the Free Software Foundation, either version 3 of the License,
   or (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */

#include "test-predicate-part1.h"

    { 0x0608, 0x0608 },
    { 0x060B, 0x060B },
    { 0x060D, 0x060D },
    { 0x061B, 0x064A },
    { 0x066D, 0x066F },
    { 0x0671, 0x06D5 },
    { 0x06E5, 0x06E6 },
    { 0x06EE, 0x06EF },
    { 0x06FA, 0x0710 },
    { 0x0712, 0x072F },
    { 0x074B, 0x07A5 },
    { 0x07B1, 0x07BF },
    { 0x0860, 0x086A },
    { 0x0870, 0x088F },
    { 0x08A0, 0x08C9 },
    { 0x2065, 0x2065 },
    { 0xFB50, 0xFBC2 },
    { 0xFBD3, 0xFD3D },
    { 0xFD50, 0xFD8F },
    { 0xFD92, 0xFDC7 },
    { 0xFDF0, 0xFDFC },
    { 0xFE1A, 0xFE1F },
    { 0xFE53, 0xFE53 },
    { 0xFE67, 0xFE67 },
    { 0xFE6C, 0xFEFE },
    { 0x10D00, 0x10D23 },
    { 0x10EC2, 0x10EC7 },
    { 0x10F30, 0x10F45 },
    { 0x10F51, 0x10F59 },
    { 0x1EC71, 0x1ECB4 },
    { 0x1ED01, 0x1ED3D },
    { 0x1EE00, 0x1EE03 },
    { 0x1EE05, 0x1EE1F },
    { 0x1EE21, 0x1EE22 },
    { 0x1EE24, 0x1EE24 },
    { 0x1EE27, 0x1EE27 },
    { 0x1EE29, 0x1EE32 },
    { 0x1EE34, 0x1EE37 },
    { 0x1EE39, 0x1EE39 },
    { 0x1EE3B, 0x1EE3B },
    { 0x1EE42, 0x1EE42 },
    { 0x1EE47, 0x1EE47 },
    { 0x1EE49, 0x1EE49 },
    { 0x1EE4B, 0x1EE4B },
    { 0x1EE4D, 0x1EE4F },
    { 0x1EE51, 0x1EE52 },
    { 0x1EE54, 0x1EE54 },
    { 0x1EE57, 0x1EE57 },
    { 0x1EE59, 0x1EE59 },
    { 0x1EE5B, 0x1EE5B },
    { 0x1EE5D, 0x1EE5D },
    { 0x1EE5F, 0x1EE5F },
    { 0x1EE61, 0x1EE62 },
    { 0x1EE64, 0x1EE64 },
    { 0x1EE67, 0x1EE6A },
    { 0x1EE6C, 0x1EE72 },
    { 0x1EE74, 0x1EE77 },
    { 0x1EE79, 0x1EE7C },
    { 0x1EE7E, 0x1EE7E },
    { 0x1EE80, 0x1EE89 },
    { 0x1EE8B, 0x1EE9B },
    { 0x1EEA1, 0x1EEA3 },
    { 0x1EEA5, 0x1EEA9 },
    { 0x1EEAB, 0x1EEBB }

#define PREDICATE(c) uc_is_property_bidi_arabic_right_to_left (c)
#include "test-predicate-part2.h"
