/* OpenMP declarations.

   Copyright (C) 2019-2026 Free Software Foundation, Inc.

   This file is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation, either version 3 of the
   License, or (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */

#if __GNUC__ >= 3
@PRAGMA_SYSTEM_HEADER@
#endif
@PRAGMA_COLUMNS@

#ifndef _@GUARD_PREFIX@_OMP_H

/* The include_next requires a split double-inclusion guard.  */
#ifdef _OPENMP
# @INCLUDE_NEXT@ @NEXT_OMP_H@
#endif

#ifndef _@GUARD_PREFIX@_OMP_H
#define _@GUARD_PREFIX@_OMP_H

#ifdef __cplusplus
extern "C" {
#endif

/* Initializes environment variables necessary for OpenMP.  */
extern void openmp_init (void);

#ifdef __cplusplus
}
#endif

#endif /* _@GUARD_PREFIX@_OMP_H */
#endif /* _@GUARD_PREFIX@_OMP_H */
