# Copyright (C) 2002-2026 Free Software Foundation, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

'''Gnulib - The GNU Portability Library
GNU software has a well-deserved reputation for running on many different types
of systems. While our primary goal is to write software for the GNU system, many
users and developers have been introduced to us through the systems that they
were already using.
Gnulib is a central location for common GNU code, intended to be shared among
GNU packages. GCC has libiberty, but this is hard to disentangle from the GCC
build tree.
Gnulib takes a different approach. Its components are intended to be shared at
the source level, rather than being a library that gets built, installed, and
linked against. Thus, there is no distribution tarball; the idea is to copy
files from Gnulib into your own source tree.
Gnulib also includes copies of a few files purely for convenience: the GNU
coding standards, the GNU maintainer information, the GPL and other licenses (in
Texinfo), assorted configuration scripts, and more. The goal is to provide all
the common infrastructure needed by GNU packages.'''

__copyright__ = '2002-2026 Free Software Foundation, Inc.'
__author__ = \
    [
        'Bruno Haible',
        'Paul Eggert',
        'Simon Josefsson',
        'Dmitry Selyutin',
        'Collin Funk',
    ]
__license__ = 'GNU GPLv3+'
